import h2o
import pickle
import pandas as pd
from pathlib import Path
from .downloader import ModelDownloader

class KBKIClassifier:
    """KBKI Text Classifier"""
    
    def __init__(self, model_dir="./models", auto_download=True):
        self.model_dir = Path(model_dir)
        self.downloader = ModelDownloader(model_dir)
        
        # Auto download models if they don't exist
        if auto_download and not self.downloader.check_models_exist():
            print("Models not found. Downloading...")
            self.downloader.download_models()
        
        self._load_models()
    
    def _load_models(self):
        """Load all required models and vectorizers"""
        # Initialize H2O
        if h2o.cluster() is None or not h2o.cluster().is_running():
            h2o.init()

        
        # Load main category model
        self.model_kategori = h2o.load_model(str(self.model_dir / 'modelh2o'))
        
        # Load vectorizer and feature names
        with open(self.model_dir / 'tfidf_vectorizer.pkl', 'rb') as f:
            self.vectorizer_p = pickle.load(f)
        
        with open(self.model_dir / 'feature_names.pkl', 'rb') as f:
            self.feature_names_p = pickle.load(f)
        
        # Load metadata
        with open(self.model_dir / 'models/metadata.pkl', 'rb') as f:
            self.metadata = pickle.load(f)
        
        self.model_paths = self.metadata['model_paths']
    
    def predict(self, text):
        """Predict category and subcategory for given text"""
        # Predict main category
        tfidf_p = self.vectorizer_p.transform([text]).toarray()
        df_p = pd.DataFrame(tfidf_p, columns=self.feature_names_p)
        h2o_p = h2o.H2OFrame(df_p)
        pred_p = self.model_kategori.predict(h2o_p)
        p_category = pred_p.as_data_frame().iloc[0, 0]
        
        # Predict subcategory
        if p_category not in self.model_paths:
            return {
                'category': p_category,
                'subcategory': None,
                'confidence': None,
                'message': f"Tidak ada model untuk kategori: {p_category}"
            }
        
        model_path = self.model_paths[p_category]
        
        if model_path == "single_class":
            with open(self.model_dir / f"models/{p_category}/single_class.pkl", 'rb') as f:
                single_class = pickle.load(f)
            return {
                'category': p_category,
                'subcategory': single_class,
                'confidence': 1.0,
                'message': 'kelas tunggal'
            }
        
        # Load subcategory model
        with open(self.model_dir / f"models/{p_category}/vectorizer.pkl", 'rb') as f:
            vectorizer = pickle.load(f)
        
        with open(self.model_dir / f"models/{p_category}/feature_names.pkl", 'rb') as f:
            feature_names = pickle.load(f)
        
        tfidf_vector = vectorizer.transform([text]).toarray()
        df_sub = pd.DataFrame(tfidf_vector, columns=feature_names)
        h2o_sub = h2o.H2OFrame(df_sub)
        
        model = h2o.load_model(model_path)
        prediction = model.predict(h2o_sub)
        pred_label = prediction['predict'][0, 0]
        pred_prob = float(prediction[pred_label][0, 0])
        
        return {
            'category': p_category,
            'subcategory': pred_label,
            'confidence': pred_prob,
            'message': 'Terklasifikasi' if pred_prob >= 0.50 else 'Tidak terklasifikasi (tingkat kepercayaan rendah)'
        }
    
    def predict_batch(self, texts):
        """Predict categories for multiple texts"""
        results = []
        for text in texts:
            results.append(self.predict(text))
        return results
    
    def predict_file(self, file_path, output_path=None):
        """Predict categories for texts in a file"""
        # Read file
        file_path = Path(file_path)
        ext = file_path.suffix.lower()
        
        if ext == '.csv':
            df = pd.read_csv(file_path)
        elif ext in ['.xlsx', '.xls']:
            df = pd.read_excel(file_path)
        else:
            raise ValueError("Unsupported file format. Use CSV or Excel.")
        
        # Predict
        predictions = []
        for text in df.iloc[:, 0]:
            result = self.predict(text)
            predictions.append(result)
        
        # Create output dataframe
        df['Kategori'] = [p['category'] for p in predictions]
        df['Subkategori'] = [p['subcategory'] for p in predictions]
        df['Tingkat_Kepercayaan'] = [p['confidence'] for p in predictions]
        df['Status'] = [p['message'] for p in predictions]
        
        # Save results
        if output_path is None:
            output_path = file_path.parent / f"{file_path.stem}_predicted.xlsx"
        
        df.to_excel(output_path, index=False)
        return output_path



import gdown
import os
import zipfile
from pathlib import Path

class ModelDownloader:
    """Download and setup model files"""
    
    def __init__(self, model_dir="./models"):
        self.model_dir = Path(model_dir)
        self.model_dir.mkdir(exist_ok=True)
    
    def download_models(self):
        """Download all required model files"""
        files_to_download = {
            'modelh2o': '1dgBIQj8Ey8yLJ9R2ml8C5pI7SWwtP4rY',
            'tfidf_vectorizer.pkl': '1Aw_ET5shSiI72N-ETJAA3U_DPeDv2yj-',
            'feature_names.pkl': '1p8q-l39XNZz8M6cwJUyDsUs_dPRTNOCd',
            'models.zip': '1RIgZOCVmCjS1vgK2KihdwcUjORXNSd0H'
        }
        
        for filename, file_id in files_to_download.items():
            output_path = self.model_dir / filename
            if not output_path.exists():
                print(f"Downloading {filename}...")
                gdown.download(
                    f'https://drive.google.com/uc?id={file_id}',
                    str(output_path),
                    quiet=False
                )
        
        # Extract models.zip
        models_zip = self.model_dir / 'models.zip'
        if models_zip.exists():
            print("Extracting models.zip...")
            with zipfile.ZipFile(models_zip, 'r') as zip_ref:
                zip_ref.extractall(self.model_dir)
    
    def check_models_exist(self):
        """Check if all required models exist"""
        required_files = [
            'modelh2o',
            'tfidf_vectorizer.pkl',
            'feature_names.pkl',
            'models/metadata.pkl'
        ]
        
        for file_path in required_files:
            if not (self.model_dir / file_path).exists():
                return False
        return True



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="kbki-classifier-deplearning-s", 
    version="0.1.2",
    author="deplearning",
    author_email="devinasawitrii@gmail.com",  
    description="KBKI (Klasifikasi Baku Komoditi Indonesia) Text Classifier",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/deplearning-s/kbki-classifier",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    python_requires=">=3.8",
    install_requires=[
        "h2o>=3.40.0",
        "gradio>=3.0.0",
        "pandas>=1.3.0",
        "scikit-learn>=1.0.0",
        "gdown>=4.0.0",
        "openpyxl>=3.0.0",
    ],
    entry_points={
        "console_scripts": [
            "kbki-classifier=kbki_classifier.app:main",
        ],
    },
)



"""
KBKI Classifier - Indonesian Commodity Classification
"""

__version__ = "0.1.0"
__author__ = "deplearning"

from .classifier import KBKIClassifier
from .downloader import ModelDownloader

__all__ = ["KBKIClassifier", "ModelDownloader"]



import gradio as gr
from .classifier import KBKIClassifier

class KBKIApp:
    """Gradio web interface for KBKI Classifier"""
    
    def __init__(self, model_dir="./models"):
        self.classifier = KBKIClassifier(model_dir)
    
    def predict_single(self, text):
        """Predict single text"""
        result = self.classifier.predict(text)
        pred_label = f"{result['subcategory']} ({result['message']})"
        confidence = f"{result['confidence']:.4f} ({result['confidence']*100:.2f}%)" if result['confidence'] else "N/A"
        return {pred_label: 1.0}, {confidence: 1.0}
    
    def predict_file(self, file):
        """Predict file"""
        output_path = self.classifier.predict_file(file.name)
        return str(output_path)
    
    def create_interface(self):
        """Create Gradio interface"""
        with gr.Blocks() as demo:
            gr.Markdown("## Prediksi Kode KBKI")
            
            with gr.Tab("Prediksi Satuan"):
                input_text = gr.Textbox(
                    label="Masukkan produk",
                    placeholder="Contoh: jagung"
                )
                output_pred = gr.Label(label="Prediksi Subkategori")
                output_conf = gr.Label(label="Tingkat Kepercayaan")
                btn = gr.Button("Prediksi")
                
                btn.click(
                    fn=self.predict_single,
                    inputs=input_text,
                    outputs=[output_pred, output_conf]
                )
                
                input_text.submit(
                    fn=self.predict_single,
                    inputs=input_text,
                    outputs=[output_pred, output_conf]
                )
            
            with gr.Tab("Prediksi File (Batch)"):
                file_input = gr.File(label="Upload File CSV atau Excel")
                file_output = gr.File(label="Download Hasil Excel")
                file_btn = gr.Button("Proses File")
                file_btn.click(fn=self.predict_file, inputs=file_input, outputs=file_output)
        
        return demo
    
    def launch(self, **kwargs):
        """Launch the Gradio app"""
        demo = self.create_interface()
        return demo.launch(**kwargs)

def main():
    """Main entry point for command line"""
    app = KBKIApp()
    app.launch(share=True, debug=True)

if __name__ == "__main__":
    main()
